#!/bin/bash

# Copyright 2016-2017 Nitor Creations Oy
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

set -e

CF_paramBakeRoleStack="bakery-roles"
CF_AWS__Region="eu-west-1"


if [ -n "$DEPLOY_ROLE_ARN" ]; then
  ROLE_ARN="$DEPLOY_ROLE_ARN"
else
  ROLE_ARN=$(ndt show-stack-params-and-outputs -r ${CF_AWS__Region} \
    ${CF_paramBakeRoleStack} -p deployRoleArn)
fi
ndt assume-role $ROLE_ARN
