#jinja2: newline_sequence:'\r\n'
<powershell>
$admin = [adsi]("WinNT://./administrator, user")
$admin.PSBase.Invoke("SetPassword", "{{ ansible_ssh_pass }}")
net user Administrator "{{ ansible_ssh_pass }}"
Invoke-Expression ((New-Object System.Net.Webclient).DownloadString('https://raw.githubusercontent.com/ansible/ansible/devel/examples/scripts/ConfigureRemotingForAnsible.ps1'))
winrm set winrm/config/service '@{AllowUnencrypted="true"}'
Stop-Transcript
</powershell>
<persist>true</persist>
